//////////////////////////////////////////////////////////////////////////////
//  Author : florent robinet (LAL - Orsay): robinet@lal.in2p3.fr
//////////////////////////////////////////////////////////////////////////////
#ifndef __VVetoPerf__
#define __VVetoPerf__

#include <TriggerPlot.h>
#include <ffl.h>
#include "VPconfig.h"

using namespace std;

/**
 * Store VetoPerf parameters.
 * \author    Florent Robinet
 */
class VP{

  friend class VetoPerf;

 private:
  VP(const string vdef, const bool vchannel, const int ncat);///< Constructor
  virtual ~VP(void);             ///< Destructor
  void Reset(void);              ///< reset veto parameters 
  bool vetochannel;              ///< true if veto channel, false if veto file
  string vetofile;               ///< veto file
  string vetoname;               ///< veto name
  string vetodir;                ///< veto directory
  double dt;                     ///< veto dead time [s]
  int n_seg;                     ///< number of veto active segments
  int n_cat;                     ///< number cluster categories
  int *n_seguse;                 ///< number of used veto segments
  int *n_cveto;                  ///< number of vetoed cluster

};

/**
 * Measure the performance of vetoes against a set of triggers.
 * \author    Florent Robinet
 */
class VetoPerf{

 public:

  /**
   * @name Constructors and destructors
   @{
  */
  /**
   * Constructor of the VetoPerf class.
   */
  VetoPerf(const vector <double> aSnrThr);

  /**
   * Destructor of the VetoPerf class.
   */
  virtual ~VetoPerf(void);
  /**
     @}
  */

  /**
   * Sets ffl.
   * Provide the ffl object which will be used to extract veto channel data.
   * @param aFfl ffl object.
   */
  inline void SetFfl(ffl *aFfl=NULL){ ffldata=aFfl; };

  /**
   * Sets the flag to use (or not) second-based veto segments.
   * If the flag is set to true, veto segments are rounded to integer values. See also Segments:MakeSecond().
   * @param aUseSec flag to use (or not) second-based veto segments.
   */
  inline void UseSecondSegments(const bool aUseSec=false){ usesec=aUseSec; };

  /**
   * Sets vetoes.
   * Returns the number of vetoes. Note: all previous vetoes are deleted.
   * @param aVetoFilePattern veto segment file pattern
   * @param aVetoChannelFile veto file listing veto channels over 1 column (patterns can be used)
  */
  int SetVetoes(const string aVetoFilePattern="", const string aVetoChannelFile="");

  /**
   * Run veto performance.
   * @param aVetoIndex veto index
  */
  bool MakeVetoPerformance(const int aVetoIndex);

  /**
   * Sets triggers and generate trigger plots.
   * The input TriggerPlot structure must verify some conditions which must be met by the user:
   * - Triggers must be clustered before calling this function.
   * - Clusters tagged with a negative value will be ignored.
   * - The number of TriggerPlot collections must match the number of SNR thresholds provided in the constructor.
   *
   * Optionnally, a Segments structure can be provided. Clusters outside the segments will be ignored (tag to -1). If a pointer to NULL is given, all the triggers segments will be considered. Similarly, clusters outside the frequency range set with SetTriggerMinimumFrequency() and SetTriggerMaximumFrequency() will be de-activated (tag to -1).
   * @param aTriggers TriggerPlot structure
   * @param aInSeg input segments for trigger selection
  */
  bool SetTriggers(TriggerPlot* aTriggers, Segments *aInSeg=NULL);

  /**
   * Sets path to output directory.
   * @param aOutdir path to output directory
  */
  bool SetOutputDirectory(const string aOutdir);

  /**
   * Make html report.
  */
  bool MakeReport(void);

  /**
   * Adds a subtitle in the html output.
   * @param SetSubTitle subtitle string
   */ 
  inline void SetSubTitle(const string aSubTitle){ subtitle=aSubTitle; };

  /**
   * Returns the number of SNR thresholds.
  */
  inline int GetNSnrThr(void){ return (int)snrthr.size(); };

  /**
   * Returns the number of vetoes.
  */
  inline int GetNvetoes(void){ return nveto; };

  /**
   * Sets minimum frequency for the trigger set.
   * @param aFrequencyMin minimum frequency [Hz]
   */
  inline void SetTriggerMinimumFrequency(const double aFrequencyMin){ freqmin=aFrequencyMin; };

  /**
   * Sets maximum frequency for the trigger set.
   * @param aFrequencyMax maximum frequency [Hz]
   */
  inline void SetTriggerMaximumFrequency(const double aFrequencyMax){ freqmax=aFrequencyMax; };


 private:
  time_t timer_start;                     ///< timer start
  string subtitle;                        ///< subtitle
  string outdir;                          ///< output directory
  Segments *InSeg;                        ///< input segments
  bool usesec;                            ///< flag to use second segments

  int nveto;                              ///< number of vetoes
  VP **vp;                                ///< vetoperf stuctures

  string TriggerFilePattern;              ///< trigger file pattern
  TriggerPlot *TP;                        ///< triggers (external, do not delete)
  double freqmin;                         ///< minimum frequency
  double freqmax;                         ///< maximum frequency
  ffl *ffldata;                           ///< ffl object  (external, do not delete)
  int n_c;                                ///< number of active clusters
  int *n_c_snr;                           ///< number of active clusters in SNR categories
  TH1D *h1_snr;                           ///< trigger snr distribution
  
  vector <double> snrthr;                 ///< SNR thresholds
  void MakeCollections(TriggerPlot *aTP); ///< make collection plots
  Segments* GetSegmentsFromChannel(const string aChannelName); ///< returns segments from a veto channel
  vector <int> Sort(void);                ///< sort veto indices

  ClassDef(VetoPerf,0)  
};


#endif


