//////////////////////////////////////////////////////////////////////////////
//  Author : florent robinet (LAL - Orsay): robinet@lal.in2p3.fr
//////////////////////////////////////////////////////////////////////////////
#include "VVetoPerf.h"
#include "OmicronUtils.h"

int printhelp(void){
  cerr<<endl;
  cerr<<"Usage:"<<endl; 
  cerr<<endl;
  cerr<<"vetoperf.exe channel=[channel name] \\"<<endl;
  cerr<<"             triggers=[trigger file pattern] \\"<<endl;
  cerr<<"             gps-start=[GPS start] \\"<<endl;
  cerr<<"             gps-end=[GPS end] \\"<<endl;
  cerr<<"             vetoes=[veto file pattern] \\"<<endl;
  cerr<<"             channel-file=[veto channel file path] \\"<<endl;
  cerr<<"             ffl=[path to ffl] \\"<<endl;
  cerr<<"             segments=[segment file] \\"<<endl;
  cerr<<"             outdir=[output directory] \\"<<endl; 
  cerr<<"             snr-thresholds=[list of SNR thresholds] \\"<<endl; 
  cerr<<"             freq-min=[minimum frequency] \\"<<endl; 
  cerr<<"             freq-max=[maximum frequency] \\"<<endl;
  cerr<<"             cluster-dt=[clustering dt] \\"<<endl;
  cerr<<"             use-sec-seg=[flag for second segments]"<<endl; 
  cerr<<endl;
  cerr<<endl;
  cerr<<"[channel name]             channel name used to retrieve centralized Omicron triggers"<<endl;
  cerr<<"[trigger file pattern]     trigger file pattern (required)"<<endl;
  cerr<<"[GPS start]                starting GPS time (integer only)"<<endl;
  cerr<<"[GPS end]                  stopping GPS time (integer only)"<<endl;
  cerr<<"[veto file pattern]        veto file pattern (required)"<<endl;
  cerr<<"[veto channel file path]   file listing the veto channels (one column)"<<endl;
  cerr<<"[path to ffl]              path to ffl for veto channels (default: /virgoData/ffl/raw.ffl)"<<endl;
  cerr<<"[segment file]             path to segment file (default: trigger segments)"<<endl;
  cerr<<"[output directory]         path to output directory (default: current)"<<endl;
  cerr<<"[list of SNR thresholds]   list of SNR thresholds. By default, snr-thresholds=\"5;8;10;20\""<<endl;
  cerr<<"[minimum frequency]        minimum trigger frequency [Hz]. By default, use the minimum trigger frequency."<<endl;
  cerr<<"[maximum frequency]        maximum trigger frequency [Hz]. By default, use the maximum trigger frequency."<<endl;
  cerr<<"[flag for second segments] Flag to use second-based veto segments: 1=YES or 0=NO. By default, =0."<<endl;
  cerr<<"[clustering dt]            cluster dt [s]. By default = 0.1 s"<<endl;
  cerr<<endl;
  return 1;
}

int main (int argc, char* argv[]){


  //++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  //--------------              COMMAND LINE              --------------
  //++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
    
  // number of arguments
  if(argc>1&&!((string)argv[1]).compare("version")){
    PrintVersion();
    return 0;
  }
  if(argc<3) return printhelp();

  // default arguments
  string chname="";
  string TriggerFilePattern="";
  int gps_start=-1;
  int gps_end=-1;
  string VetoFilePattern="";
  string VetoChannelFile="";
  string SegmentFile="";
  string outdir=".";
  string fflpath="/virgoData/ffl/raw.ffl";
  string snrthrs="5;8;10;20";
  double freqmin=-1.0;
  double freqmax=1e20;
  bool usesec=false;
  double cdt=0.1;
  
  // loop over arguments
  vector <string> sarg;
  for(int a=1; a<argc; a++){
    sarg=SplitString((string)argv[a],'=');
    if(sarg.size()!=2) continue;
    if(!sarg[0].compare("channel"))         chname=(string)sarg[1];
    if(!sarg[0].compare("triggers"))        TriggerFilePattern=(string)sarg[1];
    if(!sarg[0].compare("gps-start"))       gps_start=atoi(sarg[1].c_str());
    if(!sarg[0].compare("gps-end"))         gps_end=atoi(sarg[1].c_str());
    if(!sarg[0].compare("vetoes"))          VetoFilePattern=(string)sarg[1];
    if(!sarg[0].compare("channel-file"))    VetoChannelFile=(string)sarg[1];
    if(!sarg[0].compare("ffl"))             fflpath=(string)sarg[1];
    if(!sarg[0].compare("segments"))        SegmentFile=(string)sarg[1];
    if(!sarg[0].compare("outdir"))          outdir=(string)sarg[1];
    if(!sarg[0].compare("snr-thresholds"))  snrthrs=(string)sarg[1];
    if(!sarg[0].compare("freq-min"))        freqmin=atof(sarg[1].c_str());
    if(!sarg[0].compare("freq-max"))        freqmax=atof(sarg[1].c_str());
    if(!sarg[0].compare("cluster-dt"))      cdt=atof(sarg[1].c_str());
    if(!sarg[0].compare("use-sec-seg"))     usesec=!!atoi(sarg[1].c_str());
  }

  if(!TriggerFilePattern.compare("")){
    if(chname.compare("")&&gps_start>0&&gps_end>0) TriggerFilePattern=GetOmicronFilePattern(chname,gps_start,gps_end);// get centralized trigger files
    else{
      cerr<<"Triggers must be specified, either with a channel name and a time range or a file pattern"<<endl;
      cerr<<"Type vetoperf.exe for help"<<endl;
      return 1;
    }
  }

  // check vetoes
  if(!VetoFilePattern.compare("")&&!VetoChannelFile.compare("")){
    cerr<<"A set of vetoes must be provided."<<endl;
    cerr<<"Type vetoperf.exe for help"<<endl;
    return 1;
  }

  //++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

  // snr thresholds
  sarg=SplitString(snrthrs,';');
  vector <double> snrthr;
  for(int s=0; s<(int)sarg.size(); s++) snrthr.push_back(atof(sarg[s].c_str()));
  if(!snrthr.size()){
    cerr<<"vetoperf.exe: a set of snr thresholds must be provided."<<endl;
    return 1; 
  }

  // init VetoPerf object
  VetoPerf *vp = new VetoPerf(snrthr);

  // set output directory
  if(!vp->SetOutputDirectory(outdir)) return 2;
  
  // command
  string scommand="";
  for(int a=0; a<argc; a++) scommand+=(string)argv[a]+" ";
  vp->SetSubTitle(scommand);

  // use second segments
  vp->UseSecondSegments(usesec);

  // input segments
  Segments *inseg = NULL;
  if(SegmentFile.compare("")) inseg = new Segments(SegmentFile);
  if(gps_start>0&&gps_end>0){
    if(inseg==NULL) inseg = new Segments(gps_start, gps_end);
    else inseg->Intersect(gps_start,gps_end);
  }
  if(inseg!=NULL&&!inseg->GetLiveTime()){
    cerr<<"vetoperf.exe: no analysis segments."<<endl;
    return 3;
  }
  
  // input triggers
  TriggerPlot *TP = new TriggerPlot(vp->GetNSnrThr(),TriggerFilePattern,"","STANDARD",0);
  if(!TP->GetStatus()) return 4;

  // clustering
  TP->SetClusterizeDt(cdt);
  TP->SetClusterizeSNRthr(snrthr[0]);// hard-coded
  if(!TP->Clusterize(1)) return 5;

  // frequency selection
  if(freqmin>TP->GetFrequencyMin()) vp->SetTriggerMinimumFrequency(freqmin);
  if(freqmax<TP->GetFrequencyMax()) vp->SetTriggerMaximumFrequency(freqmax);

  // input ffl
  ffl *fdata = NULL;
  if(IsTextFile(fflpath)){
    fdata = new ffl(fflpath, "STANDARD");
    fdata->LoadFrameFile(inseg->GetEnd(inseg->GetNsegments()-1));
  }
  
  // set ffl
  vp->SetFfl(fdata);
  
  // set triggers
  if(!vp->SetTriggers(TP, inseg)) return 6;

  // set vetoes
  int nveto=vp->SetVetoes(VetoFilePattern,VetoChannelFile);
  if(nveto<=0){
    cerr<<"vetoperf.exe: no vetoes."<<endl;
    return 7;
  }
  cout<<"Number of veto files = "<<nveto<<endl;

  // loop over vetoes and measure performance
  for(int v=0; v<nveto; v++) vp->MakeVetoPerformance(v);

  // dump html report
  vp->MakeReport();
  
  delete vp;
  return 0;
}

